#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

//INPUT:
//xquery.in -- contains #rows,#cols,initial position, and map

//OUTPUT:
//query.log -- log of calls to this library

#define MAXLEN 1100
char grid[MAXLEN][MAXLEN];       //what is a good maximum size?
int curRow,curCol,initialRow,initialCol,rows,cols,headingr,headingc;
int initialized=0;
FILE *debugf;

//expecting input of the form
//5 6 2 3 S    //rows cols curRow curCol orientation    // 1 <= curRow <= rows
//X.X...                                            // 1 <= curCol <= cols
//X....X
//X....X
//X....X
//......

// direction  (headingr, headingc)
// -------------------------------
// N (-1,0)
// E (0,1)
// S (1,0)
// W (0,-1)

static char orientation(void) {
	int hr = headingr;
	int hc = headingc;
	if(hr==-1 && hc==0) return 'N';
	if(hr==0 &&  hc==1) return 'E';
	if(hr==1 &&  hc==0) return 'S';
	if(hr==0 && hc==-1) return 'W';
	else return '\?';
}

static void Init(void) {
    FILE *infp;
    int i,j,ret;
	char c;
    assert( infp = fopen( "xquery.in", "r" ) );
    assert( debugf = fopen( "query.log", "w" ) );
        fprintf(debugf, "Reading xquery.in...\n");
	fflush(debugf);
    ret=fscanf(infp, "%d %d %d %d %c ", &rows, &cols, &initialRow, &initialCol, &c );
    assert(ret==5);

    curRow = initialRow - 1;
    curCol = initialCol - 1; //change 1-based indexes to 0-based indexes
	headingr = headingc = 0;
	switch(toupper(c)) {
	//initial heading
	   case 'N': headingr = -1; break;
	   case 'S': headingr = 1;  break;
	   case 'W': headingc = -1; break;
	   case 'E': headingc = 1;  break;
	   default: 
		  fprintf(debugf, "Unrecognized orientation: \'%c\'. (Use N,S,E or W)\nUsing N\n", c);
		  fflush(debugf);
		  headingr=-1; //use north
	}

    for(i=0;i<rows;i++) fgets( grid[i], MAXLEN, infp );
    assert( grid[ curRow ][ curCol ] == '.' ); //starting position must be empty
    fclose( infp );
	fprintf(debugf,"Game board:\n");
	for(i=0;i<rows;i++) fprintf(debugf,"%s",grid[i]);
    fprintf( debugf, "Current position %d,%d facing %c\n",curRow+1,curCol+1,orientation() );
    fflush( debugf );
    initialized = 1;
}

int Forward( void ) {
//returns 1 if move is allowed, 0 if not allowed
    int newRow,newCol;
    if( !initialized ) Init();
    fprintf( debugf, "Forward()\n" );
    fflush( debugf );
    newRow = curRow+headingr;
    newCol = curCol+headingc;
    if( newRow < 0 || newCol < 0 || newRow >= rows || newCol >= cols ) {
         fprintf( debugf, "returning 0 (%d,%d is out of bounds)\n", newRow+1, newCol+1 );
         fflush( debugf );
         return 0;
    } else if( grid[ newRow ][ newCol ] != '.' ) {
         fprintf( debugf, "returning 0 (%d,%d has an obstacle)\n", newRow+1, newCol+1 );
         fflush( debugf );
         return 0;
    }
    fprintf( debugf, "returning 1 (new position is %d,%d facing %c)\n", newRow+1, newCol+1,orientation() );
    fflush( debugf );
    curRow = newRow;
    curCol = newCol;
    return 1;
}

void Right(void ) {
    int temp;
    if( !initialized ) Init();
    temp = headingr;
    headingr = headingc;
    headingc = -temp;
    fprintf( debugf, "Right(): new orientation is %c\n", orientation() );
    fflush( debugf );
}

void Left(void) {
    int temp;
    if( !initialized ) Init();
    temp = headingr;
    headingr = -headingc;
    headingc = temp;
    fprintf( debugf, "Left(): new orientation is %c\n", orientation() );
    fflush( debugf );

}

    

